/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * notificationclass.c, Copyright 2005-2006 by Adam Waldenberg
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: notificationclass.c,v 1.3 2006/09/18 23:17:29 fab Exp $
 */

#include "ambient.h"
#include "classes.h"
#include "gfx_bitmap.h"
#include "gfx_blit.h"
#include "mimetype.h"
#include "info_error_logo.h"
#include "info_help_logo.h"
#include "info_warning_logo.h"

#include <macros/vapor.h>
#include "mui_func.h"

/* public */
#include <exec/types.h>

struct Data {
	APTR bitmap, bmp_o, txt_o;
	STRPTR text;
	LONG type;
};

static void set_notificationimg(struct Data *data)
{
	APTR img;
	int w, h;

	switch (data->type)
	{
		case MV_Notification_Error:
			w = INFO_ERROR_WIDTH;
			h = INFO_ERROR_HEIGHT;
			img = &info_error;
			break;

		case MV_Notification_Warning:
			w = INFO_WARNING_WIDTH;
			h = INFO_WARNING_HEIGHT;
			img = &info_warning;
			break;

		default: /* MV_Notification_Help */
			w = INFO_HELP_WIDTH;
			h = INFO_HELP_HEIGHT;
			img = &info_help;
			break;
	}

	set(data->bmp_o, MUIA_Bitmap_Bitmap, NULL);

	if (data->bitmap)
		gfx_bitmap_delete(data->bitmap);

	if ((data->bitmap = gfx_bitmap_create(w, h, 32, BITMAPTAG_Format, BITMAPVAL_Format_ARGB32, TAG_DONE)))
	{
		gfx_blit(img, data->bitmap, BLITTAG_SrcType, BLITVAL_SrcType_Array,
			BLITTAG_SrcFormat, BLITVAL_SrcFormat_ARGB,
		TAG_DONE);

		SetAttrs(data->bmp_o,
			MUIA_Bitmap_Bitmap, gfx_bitmap_bm(data->bitmap),
			MUIA_Bitmap_Height, h,
			MUIA_Bitmap_Width, w,
			MUIA_FixHeight, h,
			MUIA_FixWidth, w,
			TAG_DONE
		);
	}
}

DEFNEW
{
	APTR txt_o = NULL, bmp_o = NULL;
	STRPTR text = NULL;
	LONG type = MV_Notification_Help;
	struct Data *data;

	FORTAG(INITTAGS)
	{
		case MA_Notification_Type:
			type = (LONG) tag->ti_Data;
			break;
		case MA_Notification_Text:
			text = (STRPTR) tag->ti_Data;
	}
	NEXTTAG

	obj = DoSuperNew(cl, obj, MUIA_Group_Horiz, TRUE,
		Child, bmp_o = BitmapObject,
			MUIA_Bitmap_Alpha, 0xffffffff,
		End,
		Child, txt_o = TextObject,
			MUIA_Text_Contents, text,
			MUIA_Text_SetMax, TRUE,
		End,
		TAG_MORE, INITTAGS
	);

	if (obj)
	{
		data = INST_DATA(cl, obj);
		data->type = type;
		data->bitmap = NULL;
		data->text = text;
		data->txt_o = txt_o;
		data->bmp_o = bmp_o;
		set_notificationimg(data);
	}

	return (ULONG) obj;
}

DEFSET
{
	GETDATA;

	FORTAG(INITTAGS)
	{
		case MA_Notification_Text:
			set(data->txt_o, MUIA_Text_Contents, tag->ti_Data);
			break;	      

		case MA_Notification_Type:
			data->type = tag->ti_Data;
			set_notificationimg(data);
			break;
	}
	NEXTTAG

	return (DOSUPER);
}

DEFDISPOSE
{
	GETDATA;

	if (data->bitmap)
		gfx_bitmap_delete(data->bitmap);

	return DOSUPER;
}

BEGINMTABLE
DECNEW
DECSET
DECDISPOSE
ENDMTABLE

DECSUBCLASS_NC(MUIC_Group, notificationclass)
